#  Copyright 2022 MTS (Mobile Telesystems)
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from typing import Any, List

import nmslib
import numpy as np


class NMTree:
    """
    KNN solution from NMSLIB.
    """

    def __init__(self, points: np.ndarray, payload: np.ndarray, ef_search: int):
        self.__was = set()
        self.__index = nmslib.init(space="l2")
        self.__index.addDataPointBatch(points)
        self.__index.createIndex()
        self.__index.setQueryTimeParams({"efSearch": ef_search})
        self.__payload = payload

    def query_batch(
        self,
        points: np.ndarray,
        payload: np.ndarray,
        out: List[List[Any]],
        group_size: int,
        amount: int = 1,
        threads: int = 1,
    ) -> None:
        """
        Write to out list indices for groups.

        Parameters
        ----------
        points : np.ndarray
            Batch of points for query
        payload : np.ndarray
            Some usable information
        out : List[List[Any]]
            Output list for indices
        group_size : int
            Sizes for groups
        amount : int, default: ``1``
            Amount groups exclude one
        threads : int, default: ``1``
            Amount of threads to be used
        """
        neighbours = self.__index.knnQueryBatch(points, k=group_size * amount, num_threads=threads)
        indices: List[np.ndarray] = [x[0] for x in neighbours]
        for i, index in enumerate(indices):
            current_answer: List = []
            for j in index:
                if self.__payload[j] in self.__was:
                    continue
                self.__was.add(self.__payload[j])
                current_answer.append(self.__payload[j])
                if len(current_answer) == amount:
                    out.append(current_answer + [payload[i]])
                    break



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': '.'}

packages = \
['ambrosia',
 'ambrosia.designer',
 'ambrosia.preprocessing',
 'ambrosia.spark_tools',
 'ambrosia.splitter',
 'ambrosia.tester',
 'ambrosia.tools',
 'ambrosia.tools._lib']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML==6.0',
 'catboost>=1.0.4,<2.0.0',
 'hyperopt>=0.2.7,<0.3.0',
 'jinja2>=3.0.0,<4.0.0',
 'joblib>=1.1.0,<2.0.0',
 'nmslib>=2.0.4,<3.0.0',
 'numpy>=1.19.5,<2.0.0',
 'pandas>=0.25.3,<2.0.0',
 'scikit-learn>=1.0.2,<2.0.0',
 'scipy>=1.6.3,<2.0.0',
 'statsmodels>=0.13.0',
 'tqdm>=4.27.0,<5.0.0']

extras_require = \
{'spark': ['pyspark>=3.2,<4.0']}

setup_kwargs = {
    'name': 'ambrosia',
    'version': '0.4.0',
    'description': 'A Python library for working with A/B tests.',
    'long_description': ".. shields start\n\nAmbrosia\n========\n\n|PyPI| |PyPI License| |ReadTheDocs| |Tests| |Coverage| |Black| |Python Versions| |Telegram Channel|\n\n.. |PyPI| image:: https://img.shields.io/pypi/v/ambrosia\n    :target: https://pypi.org/project/ambrosia\n.. |PyPI License| image:: https://img.shields.io/pypi/l/ambrosia.svg\n    :target: https://github.com/MobileTeleSystems/Ambrosia/blob/main/LICENSE\n.. |ReadTheDocs| image:: https://img.shields.io/readthedocs/ambrosia.svg\n    :target: https://ambrosia.readthedocs.io\n.. |Tests| image:: https://img.shields.io/github/actions/workflow/status/MobileTeleSystems/Ambrosia/test.yaml?branch=main\n    :target: https://github.com/MobileTeleSystems/Ambrosia/actions/workflows/test.yaml?query=branch%3Amain+\n.. |Coverage| image:: https://codecov.io/gh/MobileTeleSystems/Ambrosia/branch/main/graph/badge.svg\n    :target: https://codecov.io/gh/MobileTeleSystems/Ambrosia\n.. |Black| image:: https://img.shields.io/badge/code%20style-black-000000.svg\n    :target: https://github.com/psf/black\n.. |Python Versions| image:: https://img.shields.io/pypi/pyversions/ambrosia.svg\n    :target: https://pypi.org/project/ambrosia  \n.. |Telegram Channel| image:: https://img.shields.io/badge/telegram-Ambrosia-blueviolet.svg?logo=telegram\n    :target: https://t.me/+Tkt43TNUUSAxNWNi\n\n.. shields end\n\n.. image:: https://raw.githubusercontent.com/MobileTeleSystems/Ambrosia/main/docs/source/_static/ambrosia.png\n   :height: 320 px\n   :width: 320 px\n   :align: center\n\n.. title\n\n*Ambrosia* is a Python library for A/B tests design, split and effect measurement. \nIt provides rich set of methods for conducting full A/B testing pipeline. \n\nThe project is intended for use in research and production environments \nbased on data in pandas and Spark format.\n\n.. functional\n\nKey functionality\n-----------------\n\n* Pilots design 🛫\n* Multi-group split 🎳\n* Matching of new control group to the existing pilot 🎏\n* Experiments result evaluation as p-value, point estimate of effect and confidence interval 🎞\n* Data preprocessing ✂️\n* Experiments acceleration 🎢\n\n.. documentation\n\nDocumentation\n-------------\n\nFor more details, see the `Documentation <https://ambrosia.readthedocs.io/>`_ \nand `Tutorials <https://github.com/MobileTeleSystems/Ambrosia/tree/main/examples>`_.\n\n.. install\n\nInstallation\n------------\n\nYou can always get the newest *Ambrosia* release using ``pip``.\nStable version is released on every tag to ``main`` branch. \n\n.. code:: bash\n    \n    pip install ambrosia \n\nStarting from version ``0.4.0``, the ability to process PySpark data is optional and can be enabled \nusing ``pip`` extras during the installation.\n\n.. code:: bash\n    \n    pip install ambrosia[pyspark]\n\n.. usage\n\nUsage\n-----\n\nThe main functionality of *Ambrosia* is contained in several core classes and methods, \nwhich are autonomic for each stage of an experiment and have very intuitive interface. \n\n|\n\nBelow is a brief overview example of using a set of three classes to conduct some simple experiment.\n\n**Designer**\n\n.. code:: python\n\n    from ambrosia.designer import Designer\n    designer = Designer(dataframe=df, effects=1.2, metrics='portfel_clc') # 20% effect, and loaded data frame df\n    designer.run('size') \n\n\n**Splitter**\n\n.. code:: python\n\n    from ambrosia.splitter import Splitter\n    splitter = Splitter(dataframe=df, id_column='id') # loaded data frame df with column with id - 'id'\n    splitter.run(groups_size=500, method='simple') \n\n\n**Tester**\n\n.. code:: python\n\n    from ambrosia.tester import Tester\n    tester = Tester(dataframe=df, column_groups='group') # loaded data frame df with groups info 'group'\n    tester.run(metrics='retention', method='theory', criterion='ttest')\n\n.. develop\n\nDevelopment\n-----------\n\nTo install all requirements run\n\n.. code:: bash\n\n    make install\n\nYou must have ``python3`` and ``poetry`` installed.\n\nFor autoformatting run\n\n.. code:: bash\n\n    make autoformat\n\nFor linters check run\n\n.. code:: bash\n\n    make lint\n\nFor tests run\n\n.. code:: bash\n\n    make test\n\nFor coverage run\n\n.. code:: bash\n\n    make coverage\n\nTo remove virtual environment run\n\n.. code:: bash\n\n    make clean\n\n.. contributors\n\nAuthors\n-------\n\n**Developers and evangelists**:\n\n* `Bayramkulov Aslan <https://github.com/aslanbm>`_\n* `Khakimov Artem <https://github.com/xandaau>`_\n* `Vasin Artem <https://github.com/VictorFromChoback>`_\n",
    'author': 'Aslan Bayramkulov',
    'author_email': 'aslan.bayramkulov96@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/MobileTeleSystems/Ambrosia',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7.2,<3.11.0',
}


setup(**setup_kwargs)
